# -*- coding: utf-8 -*-

"""Top-level package for Royal Mail Rest API."""

__author__ = """Alex Hellier"""
__email__ = 'alex.hellier@gmail.com'
__version__ = '0.0.6'


# from .api import RoyalMailBaseClass
# from .tracking import TrackingApi
# from .cli import cli
